@@ -1,7 +1,6 @@
-// { dg-options "-std=gnu++0x" }
-// { dg-do compile }
+// { dg-do compile { target c++11 } }
 
-// Copyright (C) 2007-2013 Free Software Foundation, Inc.
+// Copyright (C) 2007-2020 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -21,3 +20,8 @@
 #include <memory>
 
 template class std::enable_shared_from_this<int>;
+template class std::enable_shared_from_this<const int>;
+
+class Incomplete;
+template class std::enable_shared_from_this<Incomplete>;
+template class std::enable_shared_from_this<const Incomplete>;
