@@ -1,8 +1,8 @@
-// { dg-require-namedlocale "en_US" }
+// { dg-require-namedlocale "en_US.ISO8859-1" }
 
 // 2003-10-23  Paolo Carlini  <pcarlini@suse.de>
 
-// Copyright (C) 2003-2015 Free Software Foundation, Inc.
+// Copyright (C) 2003-2020 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -30,9 +30,7 @@ void test01()
   using namespace std;
   typedef istreambuf_iterator<char> iterator_type;
 
-  bool test __attribute__((unused)) = true;
-
-  locale loc_us = locale("en_US");
+  locale loc_us = locale(ISO_8859(1,en_US));
 
   iterator_type end;
   istringstream iss;
