@@ -1,6 +1,6 @@
 // 2007-04-27  Paolo Carlini  <pcarlini@suse.de>
 
-// Copyright (C) 2007-2015 Free Software Foundation, Inc.
+// Copyright (C) 2007-2020 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -18,7 +18,8 @@
 // <http://www.gnu.org/licenses/>.
 
 // { dg-do compile }
-// { dg-error "no matching" "" { target *-*-* } 1882 }
+// { dg-prune-output "cannot convert" }
+// { dg-prune-output "no matching function .*_M_fill_assign" }
 
 #include <deque>
 
@@ -30,5 +31,7 @@ struct A
 void f()
 {
   std::deque<A> d;
-  d.assign(10, 1);
+  d.assign(10, 1);		// { dg-error "no match|here" }
 }
+
+// { dg-prune-output "iterator_traits" }
