@@ -1,6 +1,6 @@
 // 2007-04-27  Paolo Carlini  <pcarlini@suse.de>
 
-// Copyright (C) 2007-2015 Free Software Foundation, Inc.
+// Copyright (C) 2007-2020 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -18,7 +18,8 @@
 // <http://www.gnu.org/licenses/>.
 
 // { dg-do compile }
-// { dg-error "no matching" "" { target *-*-* } 1682 }
+// { dg-prune-output "cannot convert" }
+// { dg-prune-output "no matching function .*_M_fill_initialize" }
 
 #include <list>
 
@@ -31,5 +32,7 @@ void f()
 {
   typedef std::list<A> list_type;
   list_type l;
-  l.insert(l.begin(), 10, 1);
+  l.insert(l.begin(), 10, 1);	// { dg-error "here|no match" }
 }
+
+// { dg-prune-output "iterator_traits" }
