@@ -1,4 +1,4 @@
-// Copyright (C) 2011-2018 Free Software Foundation, Inc.
+// Copyright (C) 2011-2020 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -25,5 +25,7 @@
 // libstdc++/50118
 template class std::multiset<int, std::less<int>,
 			     __gnu_test::ExplicitConsAlloc<int> >;
+#if !defined __STRICT_ANSI__ && __cplusplus <= 201703L
 template class std::multiset<int, std::less<int>,
 			     __gnu_test::ExplicitConsAlloc<char> >;
+#endif
