@@ -1,6 +1,6 @@
 // { dg-require-namedlocale "en_US.UTF-8" }
 
-// Copyright (C) 2003-2013 Free Software Foundation, Inc.
+// Copyright (C) 2003-2020 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -23,24 +23,24 @@
 #include <locale>
 #include <testsuite_hooks.h>
 
-void test03()
+void test01()
 {
   using namespace std;
-  bool test __attribute__((unused)) = true;
 
   wfilebuf fb;
   locale loc(locale("en_US.UTF-8"));
   fb.pubimbue(loc);
-  fb.open("tmp_11305-3", ios_base::out);
+  fb.pubsetbuf(0, 0);
+  fb.open("tmp_11305-1", ios_base::out);
   wfilebuf::int_type n1 = fb.sputc(0x20000000);
   wfilebuf* f = fb.close();
-	
+  
   VERIFY( n1 != wfilebuf::traits_type::eof() );
   VERIFY( f );
 }
 
 int main()
 {
-  test03();
+  test01();
   return 0;
 }
