@@ -1,4 +1,7 @@
-// Copyright (C) 2003-2017 Free Software Foundation, Inc.
+// Derived from libstdc++/12048 by LJR <ljrittle@acm.org> with
+// reminder from Petur Runolfsson <peturr02@ru.is>.
+
+// Copyright (C) 2003-2020 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -15,9 +18,11 @@
 // with this library; see the file COPYING3.  If not see
 // <http://www.gnu.org/licenses/>.
 
+// { dg-require-fileio "" }
+
 #include <iostream>
-#include <cstdio>
 #include <cwchar>
+#include <cstdio>
 #include <testsuite_hooks.h>
 
 void
@@ -25,13 +30,12 @@ test01()
 {
   VERIFY( std::freopen("cin_unget-1.txt", "r", stdin) );
 
-  wchar_t c1;
-  std::wint_t c2;
-  std::wcin.get(c1);
-  std::wcin.unget();
-  VERIFY( std::wcin.good() );
-  c2 = std::fgetwc(stdin);
-  VERIFY( c2 == std::char_traits<wchar_t>::to_int_type(c1) );
+  wchar_t buf[2];
+  VERIFY( std::wcin.rdbuf()->sgetn(buf, 2) == 2 );
+  std::wint_t c1 = std::wcin.rdbuf()->sungetc();
+  std::wint_t c2 = std::wcin.rdbuf()->sbumpc();
+  VERIFY( c1 == std::char_traits<wchar_t>::to_int_type(buf[1]) );
+  VERIFY( c2 == c1 );
 }
 
 int main(void)
